// Learn cc.Class:
//  - [Chinese] http://docs.cocos.com/creator/manual/zh/scripting/class.html
//  - [English] http://www.cocos2d-x.org/docs/creator/en/scripting/class.html
// Learn Attribute:
//  - [Chinese] http://docs.cocos.com/creator/manual/zh/scripting/reference/attributes.html
//  - [English] http://www.cocos2d-x.org/docs/creator/en/scripting/reference/attributes.html
// Learn life-cycle callbacks:
//  - [Chinese] http://docs.cocos.com/creator/manual/zh/scripting/life-cycle-callbacks.html
//  - [English] http://www.cocos2d-x.org/docs/creator/en/scripting/life-cycle-callbacks.html



/* The Origin reset issue occurs if you
    1) Uncomment line 130
    2) Input a value greater than 0 in the properties panel for minMoveSpeed
*/

const GameCamera = require('GameCamera');

cc.Class({
   extends: cc.Component,

   properties: {  
	   camera: GameCamera,
	   jumpHeight: 0,
	   jumpDuration: 0,
	   jumpSpeed: 200,
	   minMoveSpeed: 0,
	   maxMoveSpeed: 0,
	   accel: 0,
   },
   
   onLoad: function () {
		
        // initialize jump action
 
		 // switch of acceleration direction
        this.accLeft = false;
        this.accRight = false;
		this.oneJump = false;
		
		this.body = this.getComponent(cc.RigidBody);

        // current horizontal speed of main character
        this.xSpeed = 0;
        this.setInputControl();
    },
   
    setJumpAction: function () {
        // jump up
        var jumpUp = cc.moveBy(this.jumpDuration, cc.p(0, this.jumpHeight)).easing(cc.easeCubicActionOut());
        // jump down
        var jumpDown = cc.moveBy(this.jumpDuration, cc.p(0, -this.jumpHeight)).easing(cc.easeCubicActionIn());
        // repeat
        return cc.repeat(cc.sequence(jumpUp, jumpDown), 1);
    },
	
		
	 setInputControl: function () {
        var self = this;
        // add keyboard event listener
        // When there is a key being pressed down, judge if it's the designated directional button and set up acceleration in the corresponding direction
        cc.systemEvent.on(cc.SystemEvent.EventType.KEY_DOWN, function (event){
            switch(event.keyCode) {
                case cc.KEY.a:
				case cc.KEY.left:
                    self.accLeft = true;
                    break;
                case cc.KEY.d:
				case cc.KEY.right:
                    self.accRight = true;				
                    break;
				case cc.KEY.w:
				case cc.KEY.up:
                    self.oneJump = true;
				    break;
            }
        });

        // when releasing the button, stop acceleration in this direction
        cc.systemEvent.on(cc.SystemEvent.EventType.KEY_UP, function (event){
            switch(event.keyCode) {
                case cc.KEY.a:
				case cc.KEY.left:
                    self.accLeft = false;
                    break;
                case cc.KEY.d:
				case cc.KEY.right:
                    self.accRight = false;
                    break;
				case cc.KEY.w:
				case cc.KEY.up:
				    self.oneJump = false;
				    break;
            }
		});
    },
	
	onBeginContact: function (contact, selfCollider, otherCollider) {
		let anim = this.getComponent(cc.Animation);
		var worldManifold = contact.getWorldManifold();
        //var points = worldManifold.points;
        //var normal = worldManifold.normal;
        if (otherCollider.node.y >= selfCollider.node.y || otherCollider.node.name === "Spike") {
		   switch(otherCollider.node.name) {
			  case "Block":
		      case "Spike":
			  case "Wall":
			    //anim.play('crash');
				//anim.stop('walk', 'jump');
	        	this.node.stopAllActions();
		        this.maxMoveSpeed = 0;
		      default: //anim.play('crash');
			           //anim.stop();
					   this.maxMoveSpeed = 0;
		}
		}

		//anim.stop('walk');
    },
	
	onEndContact: function (contact, selfCollider, otherCollider) {
		   // this.accLeft = true;
			//this.accRight = true;
	},
	
	update: function (dt) {
		
		
		//this.xSpeed = this.body.linearVelocity;
		
		
		//var anim = this.getComponent(cc.Animation);
        // update speed of each frame according to the current acceleration direction
        if (this.accLeft) {
            this.xSpeed -= this.accel * dt;
        } else if (this.accRight) {
            this.xSpeed += this.accel * dt;
			//anim.play('walk');
        } else if (this.oneJump) {
		    this.jumpAction = this.setJumpAction();
            this.node.runAction(this.jumpAction);
			//anim.play('jump');
		}
        // restrict the movement speed of the main character to the maximum movement speed
        if ( Math.abs(this.xSpeed) > this.maxMoveSpeed ) {
            // if speed reaches its limit, use the max speed with current direction
            this.xSpeed = this.maxMoveSpeed * this.xSpeed / Math.abs(this.xSpeed);
        } 
		// restrict speed of main character to minimum mvmt speed (NOTE: The problem occurs here when this property is given a value other than 0)
		if ( Math.abs(this.xSpeed) < this.minMoveSpeed ) {
			this.xSpeed = this.minMoveSpeed / this.xSpeed * Math.abs(this.xSpeed);
		}

        // update the position of the main character according to the current speed
        this.node.x += this.xSpeed * dt;
		//this.body.linearVelocity = this.speed;
    }
});